function initDevice() {
	
	/**
	 * When testing in the browser we don't have the device object, so create one
	 */
	if (!window['device']) {
		EMULATED = true;

		window.device = {
			uuid: '00000000-0000-0000-0000-000000000000',
			name: window.navigator.userAgent,
			platform: 'Browser'
		};

	} else {
		EMULATED = false;

	}

	if (!window.plugins || !window.plugins.orientationchanger) {
		window.plugins = {
			orientationchanger: {
				lockOrientation: function(){}
			}
		};
	}

	window.plugins.orientationchanger.lockOrientation('landscape');

	App.initSounds({
		points: 'points.mp3',
		zoomout: 'zoomout2.mp3',
		click: 'click.mp3',
		click2: 'click2.mp3',
		correct_answer: 'correct_answer.mp3',
		incorrect_answer: 'incorrect_answer.mp3',
		bonus_interstitual: 'bonus_interstitual.mp3',
		life_lost: 'life_lost.mp3'
	});
}


/**
 * Workaround a phonegap bug that causes the app not to load
 * the HTML/JS when it wakes up. Just quit the whole app instead
 */
document.addEventListener("backbutton", function onDeviceReady(e){
	e.preventDefault();
	navigator.app.exitApp();
}, false);


window.addEventListener('message', function(event) {
	var data;

	try {
		data = JSON.parse(event.data);
	} catch (err) {}

	if (!data) {
		return;
	}

	switch(data.method){
		case 'openUrl':
			if(typeof device != 'undefined' && device.platform === 'Android') {
				navigator.app.loadUrl(data.args.url, {openExternal:true});
			} else {
				window.open(data.args.url, '_system');
			}
		break;
	}
}, false);

var BaseApp = Backbone.Model.extend({
	defaults: {
		points: 0,
		points_game: 0,
		bonus_lives: 6,
		update_ready: false
	},

	initialize: function() {
		this.user = new User();
		this.user.loadLocalUser();

		this.user.refresh();
		
		//alert( 'Your screen resolution is ' + screen.width + 'x' + screen.height );
	},

	apiCall: function(method, params) {
		params = params || {};

		var url = 'game/' + window.config.game_id + '/' + method;

		if (quiz_game.model.get('session_id'))
			params.session_id = quiz_game.model.get('session_id');

		return App.callApi(url, params);

		var url = window.config.api_path + 'game/' + window.config.game_id + '/' + method;

		if (quiz_game.model.get('session_id'))
			params.session_id = quiz_game.model.get('session_id');

		params.device_id = window.device.uuid;

		return $.getJSON(url, params);
	},
		
	addPoints: function(points) {
		var p = this.get('points');
		this.set('points', p + parseInt(points, 10));
		console.log('Points:', this.get('points'));
	},
	
	addPointsGame: function(points_game) {
		var pg = this.get('points_game');
		this.set('points_game', pg + parseInt(points_game, 10));
		console.log('PointsGame:', this.get('points_game'));
	},

	setPoints: function(points) {
		this.set('points', parseInt(points, 10));
	},
	
	setPointsGame: function(points_game) {
		this.set('points_game', parseInt(points_game, 10));
	},

	setBonusLives: function(lives) {
		window.localStorage.setItem('lives_left',lives);
		console.log('setting lives to ' + lives);
	},
	
	resetApp: function() {
		this.setPoints(0);
		this.setPointsGame(0);
		this.setBonusLives(6);
	},

	callApi: function(method, params) {
		params = params || {};

		var url = window.config.api_path + method;
		var cookie = window.localStorage.getItem('api_cookie');

		params.device_id = window.device.uuid;

		var req = $.ajax({
			type: 'GET',
			url: url,
			data: params,
			dataType: 'json',
			xhrFields: {
				withCredentials: true
			},
			crossDomain: true,
			beforeSend: function(xhr) {
				//xhr.setRequestHeader("Cookie", cookie||'');
			}
		});

		//req.done(function(data, text_status, jqxhr) {
		//});

		return req;
	},

	loaderShow: function() {
		var self = this;

		if (this._loader_tmr) {
			clearTimeout(this._loader_tmr);
			delete this._loader_tmr;
		}

		if (!self.loader)
			self.loader = new Loader();

		// Only show the loader if it takes longer than .5 seconds
		this._loader_tmr = setTimeout(function() {
			$('body').append(self.loader.$el);
			self.loader.$el.show();
		}, 500);
	},

	loaderHide: function() {
		console.log('removing loader');
		if (!this.loader)
			return;

		var self = this;
		this.loader.$el.fadeOut(200, function() {
			self.loader.remove();
		});

		if (this._loader_tmr) {
			clearTimeout(this._loader_tmr);
			delete this._loader_tmr;
		}
	},

	advertShow: function() {
		if ($('.quiz-advert').length)
			return;

		/*
		var ap_client = {};
		ap_client.lang = navigator.language ? navigator.language : navigator.browserLanguage;
		ap_client.scr_w = screen.width;
		ap_client.scr_h = screen.height;
		ap_client.url = window.location.href.substr(0,512);
		ap_client.referer = document.referrer.substr(0,512);
		ap_frame_url = 'http://ad.leadboltads.net/show_app_ad?section_id=787009631&_=1393504651480&lang='+ap_client.lang+'&scr_w='+ap_client.scr_w+'&scr_h='+ap_client.scr_h;

		if(typeof(ap_sid)!=='undefined'){
			ap_frame_url+='&sid='+encodeURIComponent(ap_sid);
		}

		if(typeof(ap_gid)!=='undefined'){
			ap_frame_url+='&gid='+encodeURIComponent(ap_gid);
		}

		ap_frame_url += '&url=' + encodeURIComponent(ap_client.url) + '&referer=' + encodeURIComponent(ap_client.referer);
		ap_frame_url += '&t=' + (new Date()).getTime().toString();
		ap_iframe='<iframe id="ap_iframe" name="ap_iframe" width="100%" height="80" frameborder="0" src="'+ap_frame_url+'" marginwidth="0" marginheight="0" vspace="0" hspace="0" allowtransparency="true" scrolling="no"></iframe>';

		$('<div class="quiz-advert">' + ap_iframe + '</div>').appendTo('body');
		
		
		$('<div class="quiz-advert"><img onClick="openURL(\'https://suprtickets.com\');" id="advert" src="img/aytu-banner.gif"></div>').appendTo('body');
		*/
	},
	
	advertHide: function() {
		$('.quiz-advert').remove();
	},

	initSounds: function(sounds) {
		var that = this;

        // Get the full path to this app. Media object requires full path
        // window.location.pathname =
        // iOS: /var/mobile/Applications/{GUID}/{appname}.app/www/index.html
        // Android: /android_asset/www/index.html
        var root_path = window.location.pathname;
        root_path = root_path.substring(0, root_path.lastIndexOf('/') + 1);
        root_path = decodeURI(root_path);

        root_path += 'sounds/';

		this._sounds = {};
			//if(typeof Media === 'undefined') {
			//	alert('media not found');
			//} else {
			//	alert(sounds.join('\n'));
			//}
		_.each(sounds, function(sound_file, sound_name) {
			var MyMedia;

			if(typeof Media === 'undefined') {
				MyMedia = function(){ this.play = function(){}; };
			} else {
				MyMedia = Media;
			}

            var s = new MyMedia(root_path + sound_file,
                // success callback
                function() { console.log('sound success ' + root_path + sound_file + ' loaded'); },
                // error callback
                function(err) {
                    console.log("playAudio():Audio Error: " + err);

                    for (var i in err)
                        console.log('[SOUND ERR] ' + i.toString() + ' = ' + err[i].toString());
            });

			that._sounds[sound_name] = s;
			console.log('created sound ' + root_path + sound_file);
		});
	},
	playSound: function(sound_name) {
		if (typeof this._sounds[sound_name] === 'undefined') {
			console.log(sound_name + ' not found');
			return false;
		}

		return this._sounds[sound_name].play();
	},

	showUpdate: function() {
		alert('A new game update is pending. Please restart this app to continue.');
		this.set('update_ready', false);
	}
});

var App = new BaseApp();

var app_toolbar = new ToolbarApp({el: $('.app-toolbar')[0]});
var quiz_toolbar = new Toolbar({el: $('.quiz-toolbar')[0]});
app_toolbar.show();

app_toolbar.on('selected', function(section) {
	console.log('section', section);
	switch(section){
		case 'quizes':
			showQuizes();
			break;
		case 'home':
			showHome();
			break;
		case 'scoreboards':
			showScoreboards();
			break;
		case 'prizes':
			showPrizes();
			break;
		case 'community':
			showCommunity();
			break;
		case 'terms':
			showLegal();
			break;

		default:
			console.log('No section found for ', section);
	}
});

quiz_toolbar.listenTo(App, 'change:points', function(App, new_points){
	//quiz_toolbar.$('.points').text(new_points);
	quiz_toolbar.$('.points').countTo({to: new_points, refreshInterval: 50}).data('from', new_points);
});



var active_section = null;

function clearSection(callback) {
	if (!active_section)
		return callback();

	var active_section_tmp = active_section;
	active_section = null;
	active_section_tmp.closeSection(callback);
}

function openURL(url) {
	if(typeof device != 'undefined' && device.platform === 'Android') {
		navigator.app.loadUrl(url, {openExternal:true});
	} else {
		window.open(url, '_system');
	}
}
	
function showHome() {
	if (active_section)
		return clearSection(showHome);

	var section = new Home();
	$('.app-content').append(section.$el);

	active_section = section;
}

function showScoreboards() {
	if (active_section)
		return clearSection(showScoreboards);

	var scoreboard = new Scoreboard();
	$('.app-content').append(scoreboard.$el);

	active_section = scoreboard;
}

function showCommunity() {
	if (active_section)
		return clearSection(showCommunity);

	var community = new Community();
	$('.app-content').append(community.$el);

	active_section = community;
}

function showLegal() {
	if (active_section)
		return clearSection(showLegal);

	var legal = new Legal();
	$('.app-content').append(legal.$el);

	active_section = legal;
}

function showPrizes() {
	if (active_section)
		return clearSection(showPrizes);

	var section = new Prizes();
	$('.app-content').append(section.$el);

	active_section = section;
}

function showFlps() {
	if (active_section)
		return clearSection(showFlps);

	var section = new GetFlps();
	$('.app-content').append(section.$el);

	active_section = section;
}

function showQuizes() {
	if (active_section)
		return clearSection(showQuizes);

	App.loaderShow();
	var quiz_list = new QuizList();
	$('.app-content').append(quiz_list.$el);

	quiz_list.quizes.fetch({
		cache: window.config.cache_enabled,
		expires: 60000 * 10,
		error: function(){
			App.loaderHide();
			quiz_list.remove();
			quiz_list = null;
			active_section = null;

			var model = new ModelMessage({
				title: 'Couldn\'t load any quizzes..',
				message: 'We couldn\'t load any quizzes. Are you connected to the internet?'
			});
			model.on('tapped', function() {
				model.remove();
				model.off();
				model = null;
				showQuizes();
			});
			model.render();
		},
		success: function() {
			App.loaderHide();
		}
	});

	quiz_list.on('quiz_selected', function(quiz) {
		console.log('starting quiz', quiz.get('title'));

		var doStart = function() {
			quiz_list.remove();
			quiz_list = null;

			startQuiz(quiz);
		};

		if (App.user.isLoggedIn()) {
			doStart();
		} else {
			App.user.doLogin(doStart);
		}
	});

	active_section = quiz_list;
}


function startQuiz(quiz) {
	
	var quiz_game = new QuizGame({quiz: quiz});

	quiz_game.model.on('change:points', function(quiz_game_model, points) {
		console.log('caught points change', points);
		App.setPoints(points);
		App.setPointsGame(0);
	});

	quiz_game.on('question', function(question, category, extra) {
		console.log('questionevent', category, extra);
		quiz_toolbar.$('.category').text('Q' + (extra.question_num+1).toString() + '. ' + category.get('label'));
	});

	quiz_game.on('categories', function(question) {
		quiz_toolbar.$('.category').text('Select a category');
	});

	quiz_game.on('round_started', function(round_started) {
		// Round 3 costs FLP.
		// TODO: This api call may return an error due to not having enough FLP. AND IT SHOULD ASK THE USER TO CONTINUE
		console.log('Starting round', round_started);
		if (round_started == 2) {
			quiz_game.apiCall('round_three')
				.done(function(data) {
					console.log('Purchase round 3 response', data);
				});

			$('body').addClass('playing-round3');
		}

		quiz_toolbar.$('.round').text('Round ' + (round_started+1).toString());
	});

	quiz_game.on('quiz_finished', function(reason) {
		var model, $message;

		if (reason === 'more_flp') {
			quiz_game.remove();
			quiz_game.dispose();

			App.resetApp();
			App.user.fetch();

			app_toolbar.show();
			showFlps();
		}

		var continueEndQuiz = function() {
			$('body').removeClass('playing-round3');

			$message.remove();
			$message = null;

			model.remove();
			model.off();
			model = null;

			quiz_game.remove();
			quiz_game.dispose();

			App.resetApp();
			App.user.fetch();

			app_toolbar.show();
		};

		var postToTimeline = function(social_network) {
			quiz_game.apiCall('share', {
				social_network: social_network,
				getmsg: true
			})
			.done(function(data) {
				var r = confirm("The following message will be posted to your timeline: \n\n" + data.msg + "\n\nAs a thank you for sharing you will be rewarded with 100 FLP's!\n\nIs this okay?");
				if (r == true) {
					quiz_game.apiCall('share', {
						social_network: social_network,
						getmsg: false,
						msgid: data.msgid
					})
					.done(function(data) {
						if (data.status == 'ok') {
							var social = '.gameover-' + social_network + '-share';
							var socialnew = '.gameover-' + social_network + '-share-thanks';
							var socialnewreward = '.gameover-' + social_network + '-share-thanks-reward';
							$(social).hide();
							$(socialnew).show();
							if (data.reward === true) {
								$(socialnewreward).show();
							}
						}
					});
				}
			});
		}
		
		$message = Backbone.View.extend({
			events: {
				'click .gameover-scoreboard': 'clickScoreboard',
				'click .gameover-playagain': 'clickPlayagain',
				'click .gameover-twitter-share': 'clickTwitterShare',
				'click .gameover-facebook-share': 'clickFacebookShare'
			},
			initialize: function() {
				this.$el.html($('#tmpl_gameover').html());
				this.$('.points_game').text(quiz_game.model.get('points_game').toString() + ' points');
				this.$('.points').html("Your total score: " + quiz_game.model.get('points').toString());
				logintype = App.user.get('social_network');
				if (logintype && logintype !== "none" && logintype !== "undefined") {
					if (logintype === 'twitter') {
						var social = '.gameover-' + logintype + '-share';
						this.$(social).show();
					}
				}
			},
			clickScoreboard: function(event) {
				continueEndQuiz();
				showScoreboards();
			},
			clickPlayagain: function(event) {
				continueEndQuiz();
				showQuizes();
			},
			clickTwitterShare: function(event) {
				postToTimeline('twitter');
			},
			clickFacebookShare: function(event) {
				postToTimeline('facebook');
			}
		});
		
		$message = new $message();
		model = new ModelMessage({
			view: $message.$el,
			background: '#0084d7'
		});
		model.on('tapped', function() {
			//continueEndQuiz();
		});
		model.render();

		//delay(500000, continueEndQuiz);
	});

	App.loaderShow();
	quiz_game.start(function(err, start_the_game) {
		App.loaderHide();

		if (!err) {
			delay(function() {
				quiz_toolbar.show();

			}).delay(700, function() {
				quiz_game.$el.appendTo($('body'));
				start_the_game();

				// Force to end of round 2
				//quiz_game.quiz_cats.current_round = 1;
				// Force the bonus round
				//quiz_game.startBonusRound();
			});

			return;
		}

		console.log('Couldn\'t start quiz because:', err);
		quiz_game.remove();
		quiz_game.dispose();

		switch(err){
			case 'not_logged_in':
				App.user.doLogin(showQuizes);
				break;

			default:
				var display_messages = {
					'network_error': 'Do you have an internet connection?',
					'invalid_device': 'Your phone or tablet cannot be recognised'
				};

				var model = new ModelMessage({
					title: 'We couldn\'t start your quiz!',
					message: display_messages[err] || err
				});
				model.on('tapped', function() {
					model.remove();
					model.off();
					model = null;
					showQuizes();
				});
				model.render();
		}
	});
}

function showSplash(callback_fn) {
	//alert(screen.width+'x'+screen.height);
	$('.splash').fadeIn('slow');
	setTimeout(function() {
		$('.splash').fadeOut('slow');
		callback_fn();
	}, 3000);
}
