var BonusRound = (function() {

    return Backbone.View.extend({
        className: 'bonus-round',

        events: {
            'click .keyboard-key': 'onKeyClick'
        },

        initialize: function() {
            var self = this;
			this.game_over = false;
            if (window.localStorage.getItem('lives_left')) {
				this.lives = window.localStorage.getItem('lives_left');
				if (this.lives < 1) {
					this.trigger('end', {reason: 'win', points: 0});
				}
				console.log('Lives left found: ' + this.lives);
			} else {
				this.lives = 6;
				console.log('Lives not found');
			}
            this.timer = new QuestionTimer();
			this.render();
			this.updateLives();
        },

        render: function() {
            this.$el.html($('#tmpl_bonus_round').html());
			this.$el.prepend(this.timer.$el);
			this.timer.render();
			this.buildKeyboard();

			quiz_toolbar.$('.category').text('Fill in the song title');
			$('.bonus_wrapper').hide();
			$('.bonus_interstitual').fadeIn('fast').delay(2000).fadeOut('fast');
			$('.bonus_wrapper').hide().delay(2000).fadeIn('fast');
			App.advertHide();
			App.playSound('bonus_interstitual');
        },

        startTimer: function() {
            var that = this;
            var time_started = new Date();
            var time_total = window.config.bonus_length;

            this.tmr = setInterval(function() {
                var perc = ((new Date()).getTime() - time_started.getTime()) / time_total * 100;
                if (perc >= 100) {
                    clearInterval(that.tmr);
                    that.tmr = null;

                    /*that.timeoutGame();*/
					that.trigger('points', {points: 0});
					this.$('.keyboard').hide();
					this.$('.bonus-lives').hide();
					that.trigger('end', {reason: 'win', points: 0});
                }

                that.timer.setPercent(perc);
            }, 50);
        },

        timeoutGame: function() {
            this.trigger('end', {reason: 'time_up'});
        },

        buildKeyboard: function() {
            var key_width = 10;

            // [padding_from_left, keys_as_a_string]
            var key_rows = [
                [0, 'qwertyuiop'],
                [key_width/2, 'asdfghjkl'],
                [key_width + (key_width/2), 'zxcvbnm']
            ];

            var $keyboard = this.$('.keyboard');

            $.each(key_rows, function(idx, key_row){
                var left_padding = key_row[0];

                $.each(key_row[1].split(''), function(x_idx, key){
                    //var x = left_padding + x_idx * key_width;
                    var x = x_idx === 0 ? left_padding : 0;
                    $('<div class="box-thing keyboard-key"></div>')
                        .css({
                            'margin-left': x.toString() + '%',
                            width: key_width.toString() + '%'
                        })
                        .text(key.toUpperCase())
                        .data('key', key)
                        .appendTo($keyboard);
                });
            });

            return $keyboard;
        },

        startWord: function(word) {
            console.log('startWord()', word);

            this.current_word = word.toLowerCase();
            this.current_word_parts = word.toLowerCase().split('');
            this.used_letters = [];

            var $word = this.$current_word = $('<div></div>');

            $.each(this.current_word_parts, function(idx, letter) {
                var $letter = $('<b data-letter="'+letter.toLowerCase()+'">&nbsp;</b>');

                if (letter == ' ') {
                    $letter.addClass('space');
                } else {
                    $letter.addClass('hidden');
                }

                $word.append($letter);
            });

            this.$('.bonus-word').append($word);

            this.updateLives();
            this.startTimer();
        },


        onKeyClick: function(event) {
            
			if (this.game_over)
                return;

            // If the timer is not running, we have either run out of time
            // or already clicked an option.
            if (!this.tmr)
                return;

            var $this = $(event.currentTarget);
            var letter = $this.data('key');
			var self = this;
			
            // Already used
            if (this.used_letters.indexOf(letter) > -1)
                return;

            this.used_letters.push(letter);

            // Letter not found, lose life
            if (this.current_word_parts.indexOf(letter) == -1) {
                $this.addClass('used-incorrect');
				this.loseLife();
                return;
            }

            this.$current_word
                .find('[data-letter='+letter+']')
                .removeClass('hidden')
                .text(letter);

            $this.addClass('used-correct');
            
			App.playSound('correct_answer');
			
			// All letters found?
            if (this.$current_word.find('.hidden').length === 0) {
                App.setBonusLives(this.lives);
				this.game_over = true;
                clearInterval(this.tmr);
                this.tmr = null;

                var bpoints = this.wordValue(this.current_word);
				this.trigger('points', {points: bpoints});
                this.$('.keyboard').hide();
				this.$('.bonus-lives').hide();
				this.$('.bonus-word').append('<span class="bonus-word-correct">Well done! You earned ' + bpoints + ' bonus points</span>');
				setTimeout(function() {
					self.trigger('end', {reason: 'win', points: bpoints});
				}, 5000);
            }
        },

        loseLife: function() {
            var self = this;
			this.lives = (this.lives-1);
			App.setBonusLives(this.lives);

            this.updateLives();
            this.$('.bonus-lives')
                .addClass('animated pulse')
                .one('webkitAnimationEnd mozAnimationEnd MSAnimationEnd oanimationend animationend', function() {
                    $(this).removeClass('animated pulse');
                });

            if (this.lives <= 0) {
				App.playSound('incorrect_answer');
				/*setTimeout(function() {
					self.trigger('end', {reason:'no_lives'});
				}, 5000);
				return;
				*/
				this.game_over = true;
                clearInterval(this.tmr);
                this.tmr = null;

                this.trigger('points', {points: 0});
                this.$('.keyboard').hide();
				this.$('.bonus-lives').hide();
				self.trigger('end', {reason: 'win', points: 0});
            } else {
				App.playSound('life_lost');
			}
        },

        updateLives: function() {
            $lives = this.$('.bonus-lives');
            $lives.text('Lives: ' + this.lives.toString());
            /*for(var i=0; i<$lives.length; i++){
                if (i<this.lives){
                    $($lives[i]).removeClass('dead');
                } else {
                    $($lives[i]).addClass('dead');
                }
            }*/
        },


        wordValue: function(word) {
            var letter_points = {
                A: 1,
                B: 3,
                C: 3,
                D: 2,
                E: 1,
                F: 4,
                G: 2,
                H: 4,
                I: 1,
                J: 8,
                K: 5,
                L: 2,
                M: 3,
                N: 1,
                O: 1,
                P: 3,
                Q: 10,
                R: 1,
                S: 1,
                T: 1,
                U: 1,
                V: 5,
                W: 5,
                X: 8,
                Y: 4,
                Z: 10
            };

            var new_points = 0;
            word.split('').forEach(function(letter) {
                letter = letter.toUpperCase();
                if (letter_points[letter])
                    new_points += letter_points[letter] * 100;
            });

            return new_points;

        },
    });

})();