var QuizCategories = (function() {

	var QuizCategory = Backbone.Model.extend({
		randQuestion: function() {
			var questions = this.get('questions'),
				unused_questions = [],
				found_question;

			_.each(questions, function(question) {
				if (!question.used) {
					unused_questions.push(question);
				}
			});

			// No available questions left? Start using them all
			if (unused_questions.length === 0) {
				unused_questions = questions;
			}

			found_question = unused_questions[Math.floor(Math.random() * unused_questions.length)];
			if (found_question) {
				found_question.used = true;
			}

			return found_question;
		},

		numUnusedQuestions: function() {
			var questions = this.get('questions'),
				unused_questions;

			_.each(questions, function(question) {
				if (!question.used) {
					unused_questions.push(question);
				}
			});

			return unused_questions.length;
		}
	});


	var QuizCategoriesC = Backbone.Collection.extend({
		model: QuizCategory,

		initialize: function(models, options) {
			this.quiz_cats = options.quiz_cats;
		},

		pickRand: function() {
			return this.models[Math.floor(Math.random() * this.models.length)];
		}
	});


	var QuizCategories = Backbone.View.extend({
		className: 'quiz-categories',

		events: {},

		tiles: null,
		current_round: 0,
		max_rounds: 2,

		current_row: 0,
		max_rows: window.config.category_rows,
		max_cols: window.config.category_cols,

		initialize: function(options) {
			this.quiz_game = options.quiz_game;
			this.tiles = [];
			this.categories = new QuizCategoriesC(undefined, {quiz_cats: this});

			/*$('<div class="quiz-advert"></div>').appendTo(this.$el);*/
		},

		loadCategories: function(cb) {
			var self = this;

			this.quiz_game.apiCall('categories', {round: this.current_round})
				.done(function(data) {
					self.categories.reset(data);
					cb(null);
				})
				.fail(function() {
					cb('Error');
				});

			/*this.categories.fetch({
				success: function() {
					cb(null);
				},

				error: function() {
					cb('Error');
				}
			});*/
		},

		drawQuestion: function() {
			console.log('drawQuestion()');
			this.$el.html($('#tmpl_next_question').html());

			var rounds = this.quiz_game.model.get('rounds');
			var points = rounds[this.current_round];
			var question_points = points[this.current_row];

			var question_points_next = points[this.current_row+1] || 'bonus';

			this.$('.quiz-next-question-meta.this .points').text(question_points.toString() + ' points');
			this.$('.quiz-next-question-meta.this .question_num').text('Question ' + (this.current_row+1).toString());

			if (question_points_next !== 'bonus') {
				this.$('.quiz-next-question-meta.next').removeClass('bonus');
				this.$('.quiz-next-question-meta.next .points').text(question_points_next.toString() + ' points');
				this.$('.quiz-next-question-meta.next .question_num').text('Question ' + (this.current_row+2).toString());
			} else {
				this.$('.quiz-next-question-meta.next').addClass('bonus');
				this.$('.quiz-next-question-meta.next .points').text('');
				this.$('.quiz-next-question-meta.next .question_num').text('');
			}

			//this.$('.quiz-this-question').transition({top: '-100%'}, 200).animate({top: '0%'}, 700);
			this.$('.quiz-this-question').addClass('animated shakesmall');

			var multipliers = [2, 3, 5, 7, 10],
				multiplier_idx;
			var tile;
			var category;

			for(var i=0; i<5; i++) {
				category = this.categories.pickRand();

				tile = new QuizCategoryTile({
					category: category,
					caption: category.get('label'),
					selectable: false,
					row: i,
					tile_idx: i,
					artist: this.quiz_game.quiz.get('title'),
				});

				tile.$el.addClass('c' + (i+1).toString());
				this.tiles.push(tile);

				multiplier_idx = Math.floor(Math.random() * multipliers.length);
				tile.score_multiplier = multipliers.splice(multiplier_idx, 1)[0];
				tile.points = question_points; //points[Math.floor(i/this.max_cols)];

				tile.render();
				tile.$el.appendTo(this.$('.quiz-next-question-multipliers-inner'));
				this.listenTo(tile, 'clicked', this.cardClicked);
			}

			_.defer(function() {
				this.$('.quiz-next-question-card').addClass('show');
			});

			App.advertShow();
		},

		cardClicked: function(card) {
			var self = this;
			var current_idx =0;

			App.playSound('click2');
			
			if (self.processing_click)
				return;

			self.processing_click = true;

			var fn = function() {
				var cat = self.tiles[current_idx];
				var card_src;
				var tip;

				if (!cat ) {
					// All tiles have been disabled. Continue with the selection
					delay(1500, function(){
						// The point counter animation
						$('.quiz-next-question-meta.this .points')
							.countTo({to: (card.score_multiplier * card.points), refreshInterval: 50})
							.css('font-size', '1.5em')
							.css('-webkit-animation-duration', '2s')
							.addClass('animated tada');
					})
					.delay(3000, function(){
						_.each(self.tiles, function(tile) {
							tile.$el.addClass('hide');
						});

						App.playSound('zoomout');
					})
					.delay(200, function(){
						delete self.processing_click;

						_.each(self.tiles, function(tile) {
							self.stopListening(tile);
							tile.remove();
						});
						self.tiles = [];

						self.trigger('category_selected', card);
					});

					return;
				}

				if (cat !== card) {
					cat.$el.find('.back img').attr('src', 'img/card_back.png?' + Math.random());
					cat.setDisabled(true);
				} else {
					cat.setSelectable(true);
					cat.setDisabled(true);
					setTimeout(function() {
						cat.setDisabled(false);
					}, 400);
				}

				cat.$el.addClass('flip');

				//cat.setSubCaption(cat.category.get('label'));
				//cat.setCaption('x' + (cat.score_multiplier || 1).toString());
				cat.setSubCaption('x' + (cat.score_multiplier || 1).toString());

				current_idx++;
				setTimeout(fn, 70);
				
			};

			fn();
			
		},

		generateCategories: function() {
			var self = this,
				cat, tile, $points_tag;

			var rounds = this.quiz_game.model.get('rounds');
			var points = rounds[this.current_round];

			this.max_rounds = rounds.length;

			var board = this.quiz_game.model.get('board');
			this.max_cols = board[0];
			this.max_rows = board[1];

			for (var i=0; i<this.max_cols*this.max_rows; i++) {
				cat = this.categories.pickRand();

				tile = new QuizCategoryTile({
					category: cat,
					caption: cat.get('label'),
					selectable: false,
					row: Math.floor(i/this.max_cols),
					tile_idx: i
				});

				tile.score_multiplier = 1;
				tile.points = points[Math.floor(i/this.max_cols)];

				tile.$el.prependTo(this.$el);
				tile.render();

				this.listenTo(tile, 'clicked', this.categorySelected);

				this.tiles.push(tile);

				// If starting a new row.. place the points tag
				if (i % this.max_cols === this.max_cols - 1) {
					$points_tag = $('<div class="points-tag"><span>' + tile.points + '</span></div>');
					var f = (100 / (this.max_cols+1));
					$points_tag.css({
						width: Math.floor((f/6)*5).toString() + '%',
						'margin-right': Math.floor((f/6)*1).toString() + '%'
					});
					this.$el.prepend($points_tag);
				}
			}

			this.$el.prepend('<div class="quiz-category-bonus-round"></div>');

			this.$el.find('.quiz-category-wrap').css('width', Math.floor(100 / (this.max_cols+1)+1).toString() + '%');
			console.log(tile.$el.find('.quiz-category-wrap'));
		},

		clearTiles: function() {
			var self = this;

			_.each(this.tiles, function(tile) {
				tile.$el.remove();
				self.stopListening(tile);
			});

			this.tiles = [];

			this.$('.quiz-category-bonus-round').remove();
		},

		selectableRow: function(row) {
			var self = this;

			_.each(this.tiles, function(tile, idx) {
				//cat.setSelectable(idxToRow(idx) == row);
				tile.setSelectable(tile.row == row);
			});

			var row_height = this.$('.quiz-category-wrap:first').outerHeight();
			var this_height = this.$el.outerHeight();
			var rows = this.max_rows;
			var row_to_show = this.current_row;
			//console.log('showing row ', row_to_show);
			//this.$el.css('top', (-this_height + (row_height * row_to_show)).toString() + 'px');

			//this.$el.css('top', (-(row_height * (this.max_rows-row_to_show)) + $(window).height() - 40).toString() + 'px');
			//this.$el.css('transform', 'translateY(' + (-(row_height * (this.max_rows-row_to_show) + this.$('.bonus-round').outerHeight()) + $(window).height() - 40).toString() + 'px' + ')');

			var new_y = (-(row_height * (this.max_rows-row_to_show) + this.$('.quiz-category-bonus-round').outerHeight() + 50) + $(window).height() - 40);
			//this.$el.css('-webkit-transform', 'translate3d(0, ' + new_y.toString() + 'px, 0)');
			this.$el.transition({y: new_y});

			App.advertShow();
		},

		categorySelected: function(category) {

			// If we already have an active category (ie. still in its transition step),
			// then don't select another
			if (this.active_category)
				return;

			// Only categories on our row are allowed
			if (category.row != this.current_row)
				return;

			// Make note of the selected category
			this.active_category = category;

			var self = this;
			var current_idx = 0;
			var multipliers = [2, 3, 5, 7, 10];
			var row_cats = _.where(this.tiles, {row: this.current_row});
			row_cats = row_cats.reverse();

			// Add the score multipliers to this row
			_.each(row_cats, function(cat) {
				var idx = Math.floor(Math.random() * multipliers.length);
				cat.score_multiplier = multipliers.splice(idx, 1)[0];
			});

			cat.$el.find('.caption').show();
			cat.$el.find('.subcaption').show();
			
			// The function cycles over the number of columns, disabling each one
			// every 50ms. Effect = swiping disabling of tiles on the current row.
			var fn = function() {
				var cat = row_cats[current_idx];

				if (!cat ) {
					// All tiles in the row have been disabled. Continue with the selection
					setTimeout(function(){
						delete self.active_category;

						category.setDisabled(true);
						self.trigger('category_selected', category);
					}, 2000);
					return;
				}

				if (cat !== category)
					cat.setDisabled(true);

				cat.setSelectable(false);
				cat.$('.quiz-category-name').text('x' + (cat.score_multiplier || 1).toString());

				current_idx++;
				setTimeout(fn, 50);
			};

			fn();
		},

		nextRound: function() {
			this.clearTiles();
			this.current_round++;
			this.current_row = 0;

			//this.generateCategories();
			//this.selectableRow(this.current_row);
			this.drawQuestion();

			this.trigger('round_started', this.current_round);
		},

		nextRow: function() {
			console.log('nextRow()');
			// Ready for bonus round?
			if (this.current_row+1 == this.max_rows && this.current_round <= this.max_rounds) {
				console.log('triggering round_finished');
				this.trigger('round_finished', this.current_round);

			}

			// Ready for game over?
			else if (this.current_row+1 == this.max_rows && this.current_round >= this.max_rounds) {
				console.log('triggering quiz_finished');
				this.trigger('quiz_finished');

			}

			// Just move to the next row
			else {
				this.current_row++;
				//this.selectableRow(this.current_row);
				this.drawQuestion();
			}
		},

		start: function() {
			this.current_round = 0;
			this.current_row = 0;
			//this.generateCategories();
			//this.selectableRow(0);
			this.drawQuestion();
			this.trigger('start');
			this.trigger('round_started', this.current_round);
		},

		dispose: function() {
			this.clearTiles();
		}
	});

	return QuizCategories;
})();