var QuizGame = (function() {

	var QuizGameM = Backbone.Model.extend({
		defaults: {
			game_id: 0,
			created_at: 0,
			points: 0,
			points_flp: 0,
			points_game: 0
		},

		initialize: function(props) {
			this.set('created_at', new Date());
		},

		// Check the server to see if we can start a quiz
		checkStart: function(cb) {
			var self = this,
				api_params = {};

			api_params.topic_id = this.get('quiz_game').quiz.get('id');

			//$.getJSON(window.config.api_path + 'game/' + window.config.game_id + '/start_session', api_params)
			this.get('quiz_game').apiCall('start_session', api_params)
				.done(function(data) {
					if (!data.session_id)
						return cb(false, data.error || 'unknown_reason');

					if (data.err) {
						/*
							err =
								user_already_playing
								banned_from_game
						*/
						return cb(false, data.err);
					}

					self.set('session_id', data.session_id);
					self.set('points_flp', data.points_flp);
					self.set('points', data.points_flp);
					self.set('board', data.board);
					self.set('rounds', data.rounds);
					self.set('bonus_words', data.bonus_words || []);
					self.set('unused_bonus_words', data.bonus_words || []);
					cb(true);

				})
				.fail(function() {
					cb(false, 'network_error');
				});
		},

		addPoints: function(points) {
			var p = parseInt(this.get('points'), 10);
			this.set('points', p + parseInt(points, 10));
		},
		
		addPointsGame: function(points_game) {
			var pg = parseInt(this.get('points_game'), 10);
			this.set('points_game', pg + parseInt(points_game, 10));
		}
	});


	return Backbone.View.extend({
		className: 'quiz',

		initialize: function(options) {
			var self = this;

			this.quiz = options.quiz;

			this.model = new QuizGameM({quiz_game: this});
			this.quiz_cats = new QuizCategories({quiz_game: this});

			this.questions_asked = 0;

			this.listenTo(this.quiz_cats, 'category_selected', this.categorySelected);
			this.listenTo(this.quiz_cats, 'round_started', this.roundStarted);
			this.listenTo(this.quiz_cats, 'round_finished', this.roundFinished);
			this.listenTo(this.quiz_cats, 'quiz_finished', this.quizFinished);

			this.listenTo(this, 'quiz_finished', function(reason) {
				var end_types = {
					completed: 'completed',
					incorrect_answer: 'incorrect_answer',
					time_up: 'time_up'
				};

				// TODO: Fill in the comments out params below
				self.apiCall('end_session', {
					type: end_types[reason] || 'completed'
					//lives: 0
				});

			});

			/* Was used to push categories up/down depending on screen orientation
			var supportsOrientationChange = "onorientationchange" in window;
			this._orientationEvent = supportsOrientationChange ? "orientationchange" : "resize";
			this._onOrientationChange = function() {
				if (self.quiz_cats.$el.is(':visible'))
					self.quiz_cats.selectableRow(self.quiz_cats.current_row);
			};
			*/
		},

		start: function(cb) {
			var self = this;

			this.model.checkStart(function(can_start, reason) {
				if (!can_start)
					return cb(reason);

				self.quiz_cats.loadCategories(function(err) {
					if (!err) {
						// The caller now know we can safely start a game..
						// Let the caller do any pre-game stuff then continue displaying the game
						cb(null, function() {
							self.quiz_cats.start();
							self.showCategories();
							//self.quiz_cats.selectableRow(0);
						});

					} else {
						cb('Error loading categories');
					}
				});
			});
		},


		dispose: function() {
			if (this.quiz_cats) {
				this.quiz_cats.remove();
				this.quiz_cats.off();
				this.quiz_cats = null;
			}

			this.model.off();
			this.model = null;

			this.off();
		},

		apiCall: function(method, params) {
			params = params || {};

			var url = 'game/' + window.config.game_id + '/' + method;

			if (this.model.get('session_id'))
				params.session_id = this.model.get('session_id');

			return App.callApi(url, params);



			var url = window.config.api_path + 'game/' + window.config.game_id + '/' + method;

			if (this.model.get('session_id'))
				params.session_id = this.model.get('session_id');

			params.device_id = window.device.uuid;

			return $.getJSON(url, params);
		},




		showCategories: function() {
			this.$el.append(this.quiz_cats.$el);
			this.quiz_cats.$el.show();

			// Slower devices can catch a bug inwhich elements are not re-painted. So we force it to
			// http://www.eccesignum.org/blog/solving-display-refreshredrawrepaint-issues-in-webkit-browsers
			this.$el.css('webkitTransform', 'scale(1)');

			this.trigger('categories');
		},

		showQuestion: function(question_obj, tile) {
			var self = this;

			var question = new QuizQuestion({question: question_obj, score_multiplier: tile.score_multiplier});
			question.render();

			this.trigger('question', question, tile.category, {question_num: this.questions_asked});

			// Build up an array of all the available answers/options
			var options = [];
			options.push(new QuizAnswer({answer: question_obj.options[0], correct: true, points: tile.points}));
			_.each(question_obj.options.slice(1), function(option) {
				options.push(new QuizAnswer({answer: option}));
			});

			// Shuffle the answers and add them to the question
			// (Comment out the below line to make A always the correct answer)
			
			options = _.shuffle(options);
			_.each(options, function(option) {
				question.answers.add(option);
			});

			this.$el.append(question.$el);
			question.startTimer();

			question.on('answer_selected', function(answer, is_correct) {
				console.log('answer selected', answer.get('points'), answer);

				self.questions_asked++;

				if (answer.get('points')) {
					App.playSound('points');
					self.model.addPoints(answer.get('points') * parseInt(tile.score_multiplier, 10));
					self.model.addPointsGame(answer.get('points') * parseInt(tile.score_multiplier, 10));
				}

				self.apiCall('question_asked', {
					round: self.quiz_cats.current_round,
					tile_x: tile.tile_x,
					tile_y: self.quiz_cats.current_row,
					multiplier: tile.score_multiplier,
					points: answer.get('points') ? answer.get('points') * parseInt(tile.score_multiplier, 10) : 0,
					question_id: question.question.id,
					is_correct: !!is_correct,

					// TODO: Add the question timers back in
					start: 0,
					length: 0
				});

				question.remove();
				question.dispose();
				question = null;

				/*
				if (is_correct) {
					// Remove!!!
					self.showCategories();
					self.quiz_cats.nextRow();
				} else {
					self.quiz_cats.remove();
					self.quiz_cats.dispose();
					self.quiz_cats = null;

					self.trigger('quiz_finished', 'incorrect_answer');
				}
				*/
				
				self.showCategories();
				self.quiz_cats.nextRow();
				
			});

			question.on('time_up', function() {
				console.log('Time up');

				/*
				question.remove();
				question.dispose();
				question = null;

				self.quiz_cats.remove();
				self.quiz_cats.dispose();
				self.quiz_cats = null;

				self.trigger('quiz_finished', 'time_up');
				*/
				
				self.questions_asked++;
				question.remove();
				question.dispose();
				question = null;
				
				self.showCategories();
				self.quiz_cats.nextRow();
			});
		},


		startBonusRound: function() {
			var self = this;

			console.log('Starting bonus round');

			this.quiz_cats.$el.hide();

			var bonus_words, bonus_word;
			bonus_words = this.model.get('unused_bonus_words');

			// If no unused bonus words are found, get all available words
			if (!bonus_words || ! bonus_words.length) {
				bonus_words = this.model.get('bonus_words');
			}

			// Get a random word from the list
			bonus_word = _.sample(bonus_words);

			// Remove our chosen word and save the list as unsed for future use
			bonus_words = _.without(bonus_words, bonus_word);
			this.model.set('unused_bonus_words', bonus_words);

			App.advertHide();
			
			var bonus_round = new BonusRound();
			bonus_round.$el.appendTo(this.$el);
			bonus_round.render();
			bonus_round.startWord(bonus_word || 'Unknown');
			
			App.advertHide();
			
			this.listenTo(bonus_round, 'end', function(event) {
				console.log('Bonus round ended due to', event.reason);

				this.stopListening(bonus_round);
				bonus_round.remove();
				bonus_round = null;

				if (event.reason !== 'win') {
					this.apiCall('bonus_played', {type: 'hangman', passed: false});

					this.quiz_cats.remove();
					this.quiz_cats.dispose();
					this.quiz_cats = null;

					this.trigger('quiz_finished', 'no_lives');

				} else {
					this.apiCall('bonus_played', {type: 'hangman', passed: true, points: event.points});

					if (this.quiz_cats.current_round == 0) {
						self.showCategories();
						self.quiz_cats.nextRound();

					} else if (this.quiz_cats.current_round == 2) {
						self.trigger('quiz_finished', 'completed');

					} else {
						// Round 3 needs user permission as it costs FLP
						var round3 = new Fullscreens.Round3();
						model = new ModelMessage({
							view: round3.$el,
							'background-image': 'url(img/bg_purple.jpg)'
						});
						model.render();
						round3.render();

						round3.on('play_round3', function() {
							// Round 3 needs user permission as it costs FLP
							var current_flps = parseInt(App.user.get('flp'));
							if (current_flps >= 1000) {
								model.remove();

								round3.off();
								round3.remove();
								round3 = model = null;

								self.showCategories();
								self.quiz_cats.nextRound();
							} else {
								alert("You do not currently have enough FLP's to unlock Round 3. Please either purchase some FLP's or play again to build up your FLP's.");
							}
						});

						round3.on('more_flp', function() {
							model.remove();

							round3.off();
							round3.remove();
							round3 = model = null;

							self.quiz_cats.remove();
							self.quiz_cats.dispose();
							self.quiz_cats = null;

							self.trigger('quiz_finished', 'more_flp');
						});

						round3.on('skip', function() {
							model.remove();

							round3.off();
							round3.remove();
							round3 = model = null;

							self.quiz_cats.remove();
							self.quiz_cats.dispose();
							self.quiz_cats = null;

							self.trigger('quiz_finished', 'skip_round3');
						});
					}
				}
			});

			this.listenTo(bonus_round, 'points', function(event) {
				this.model.addPoints(event.points);
				this.model.addPointsGame(event.points);
			});

		},


		categorySelected: function(tile) {
			var question = tile.category.randQuestion();

			App.advertHide();
			this.quiz_cats.$el.hide();
			this.showQuestion(question, tile);
		},


		roundFinished: function(round_finished) {
			this.trigger('round_finished', round_finished);

			this.startBonusRound();
		},


		roundStarted: function(round_started) {
			this.questions_asked = 0;
			this.trigger('round_started', round_started);
		},


		quizFinished: function() {
			this.quiz_cats.remove();
			this.quiz_cats.dispose();
			this.quiz_cats = null;

			this.trigger('quiz_finished', 'completed');
			App.advertHide();
		}
	});
})();