var QuizQuestion = (function() {

    var QuizAnswers = Backbone.Collection.extend({
        model: QuizAnswer
    });


    var QuizQuestion = Backbone.View.extend({
        className: 'quiz-question',
        events: {
            'click .quiz-answer': 'onAnswerClick'
        },

        initialize: function(options) {
            this.question = options.question;

            this.answers = new QuizAnswers();
            this.listenTo(this.answers, 'add', this.addedAnswer);

            this.timer = new QuestionTimer();
        },

        render: function() {
            this.$el.append(this.timer.$el);
            this.timer.render();

            var html = '<h4 class="box-thing"></h4>';
            html += '<div class="quiz-answers"></div>';

            this.$el.append(html);
            this.$('h4').text(this.question.question);
        },

        dispose: function() {
            if (this.tmr)
                clearInterval(this.tmr);

            this.timer.remove();

            this.off();
        },

        startTimer: function() {
            var that = this;
            var time_started = new Date();
            var time_total = window.config.question_length;

            this.tmr = setInterval(function() {
                var perc = ((new Date()).getTime() - time_started.getTime()) / time_total * 100;
                if (perc >= 100) {
                    clearInterval(that.tmr);
                    that.trigger('time_up');
                }

                that.timer.setPercent(perc);
            }, 50);
        },

        addedAnswer: function(answer) {
            var html = '<div class="quiz-answer-wrap">';
            html += '<div class="quiz-answer box-thing">';
            html += '<h5></h5>';
            html += '<p></p>';
            html += '</div>';
            html += '</div>';

            var $answer = $(html);

            var letters = ['A.', 'B.', 'C.', 'D.', 'E.'];
            $answer.find('p').text(answer.get('answer'));

            $answer.find('h5').text(letters[this.$('.quiz-answer-wrap').length]);

            $answer.find('.quiz-answer').data('answer', answer);
            this.$('.quiz-answers').append($answer);
        },

        onAnswerClick: function(event) {
            var that = this;

            // If the timer is not running, we have either run out of time
            // or already clicked an option.
            if (!this.tmr)
                return;

            clearInterval(this.tmr);
            this.tmr = null;

            var $target = $(event.currentTarget);
            var answer = $target.data('answer');
            var is_correct = answer.get('correct');
            var new_class = answer.get('correct') ? 'correct' : 'incorrect';

            delay(function() {
                $target.addClass(new_class);

                if (is_correct) {
                    App.playSound('correct_answer');
                } else {
                    App.playSound('incorrect_answer');
                }

            }).delay(100, function() {
                $target.removeClass(new_class);

            }).delay(70, function() {
                $target.addClass(new_class);

            }).delay(2000, function() {
                that.trigger('answer_selected', answer, answer.get('correct'));
            });
        }
    });

    return QuizQuestion;
})();