var Scoreboard = (function() {
	var Score = Backbone.Model.extend({});

	var ScoreboardC = Backbone.Collection.extend({
		model: Score,
		url: function() {
			return window.config.api_path + 'game/' + window.config.game_id + '/scores.json';		
		}
	});

	var RecentPlayersC = Backbone.Collection.extend({
		model: Score,
		url: function() {
			return window.config.api_path + 'game/' + window.config.game_id + '/recentplayers.json';
		}
	});
	
	return Backbone.View.extend({
		className: 'scoreboard',

		events: {
			'change .filter-country': 'updateFilters',
			'change .filter-topic': 'updateFilters',
			'click .scoreboard-my-position': 'onClickFindUser'
		},

		initialize: function() {
			var self = this;

			if (App.get('update_ready')) {
				App.showUpdate();
			}
			
			this.scoreboard = new ScoreboardC();
			this.recentplayers = new RecentPlayersC();
			this.listenTo(this.scoreboard, 'reset', this.renderScoreTable);
			this.listenTo(this.recentplayers, 'reset', this.renderRecentPlayers);
			this.render();

			this.countries = [];
			this.topics = [];

			App.loaderShow();
			App.callApi('game/'+window.config.game_id+'/scorestats.json')
				.done(function(data) {
					self.countries = data.countries;
					self.topics = data.topics;

					self.populateFilters();
					self.updateFilters();
				});
		},

		render: function() {
			this.$el.html($('#tmpl_scoreboard').html());
		},

		populateFilters: function() {
			var self = this;

			var $topic_select = this.$('.filter-topic');
			_.forEach(this.topics, function(topic) {
				$('<option></option>')
					.val(topic.id)
					.text(topic.title)
					.appendTo($topic_select);
			});

			var $country_select = this.$('.filter-country');
			var flag_idx = 0;
			_.forEach(this.countries, function(country, idx) {
				$('<option></option>')
					.val(country.country)
					.text(country.country)
					.appendTo($country_select);

				/*if(country.country != 'XX' && flag_idx < $top_country.length){
					var $flag = $($top_country[flag_idx]);
					$flag.css({
						'background': 'url(img/flags/'+country.country.toLowerCase()+'.png) no-repeat',
						'background-size': $flag.width().toString()+'px ' +$flag.height().toString()+'px'
					})
					.data('country', country.country);

					flag_idx++;
				}*/
			});
		},

		updateFilters: function(event) {
			this.country = this.$('.filter-country').val();
			this.topic_id = this.$('.filter-topic').val();
			this.current_user = ''
			this.current_user = App.user.get('username');
			this.loadPage(1);
		},

		onClickFindUser: function(event) {
			var current_user = App.user.get('username');
			if (current_user) {
				//$(".scoreboard-table table tbody tr:contains('" + current_user + "')").each(function(){
				$('tr[data-id="' + current_user + '"]').addClass('current-user');
				//$(".scoreboard-table").find('[data-id="' + current_user + '"]').addClass('current-user');
				//});
			}
		},

		loadPage: function(page_num) {
			App.loaderShow();
			this.scoreboard.fetch({
				data: {page: page_num, country: this.country, topic: this.topic_id, current_user: this.current_user},
				reset: true,
				success: function() { App.loaderHide(); },
				fail: function() { App.loaderHide(); }
			});
			this.recentplayers.fetch({
				data: {},
				reset: true
			});
		},

		renderScoreTable: function() {
			var self = this;
			var topics = this.topics;

			this.$('.scoreboard-table').empty();

			var $table = $('<table></table>');
			var $tmpl = $('<tr><td class="pos"></td><td class="name"></td><td class="country"></td><td class="topic"></td><td class="score"></td></tr>');

			this.scoreboard.each(function(score) {
				var title,
				$row = $tmpl.clone();

				$row.find('.pos').text(score.get('pos'));
				$row.find('.name').parent('tr').attr('data-id', score.get('name'));
				$row.find('.name').text(score.get('name'));
				$row.find('.country').append($('<img />').attr('src', 'img/flags/' + score.get('country').toLowerCase() + '.png'));
				//$row.find('.topic').text(self.getTopicById(score.get('topic')).title);
				title = self.getTopicById(score.get('topic')).title;
				$row.find('.topic').append($('<img />').attr('src', 'img/artists/' + title.replace(/[^a-zA-Z]/g, '').toLowerCase() + '/scoreboard.png'));

				$row.find('.score').text(score.get('score'));
				
				current_user = App.user.get('username');
				if (current_user && current_user != "") {
					if (score.get('name') == App.user.get('username')) {
						//$row.addClass('current-user');
					}
				}
				
				$table.append($row);
			});

			this.$('.scoreboard-table').append($table);
		},
		
		renderRecentPlayers: function() {
			console.log("recent");
			var self = this;

			this.$('.scoreboard-recent-players').empty().html('<p>Recent players</p>');

			this.recentplayers.each(function(player) {
				this.$('.scoreboard-recent-players').append($('<img />').attr('src', player.get('avatar_url')));
			});
		},

		closeSection: function(callback) {
			this.remove();
			callback();
		},

		getTopicById: function(id) {
			var models = _.where(this.topics, {id: id});

			return models.length === 0 ?
				null :
				models[0];
		}
	});

})();