window.launchGame = function() {
    var deviceready = false;
    var scripts_loaded = false;
    var startup_time = new Date();

    function doStart() {
        $(function() {
            FastClick.attach(document.body);

            initDevice();
            showSplash(function() {
                showHome();
            });
        });
    }


    loadScripts().then(function() {
        scripts_loaded = true;

        if ((scripts_loaded && deviceready) || window.location.hash) {
            doStart();
        }

    }, function (error) {
        // There was an error fetching the script
        console.log(error, error.message, error.stack);
    });

    document.addEventListener('deviceready', function() {
        deviceready = true;

        if (scripts_loaded && deviceready)
            doStart();
    }, false);




    function checkUpdate() {
        var url = 'http://aytugames.com/gameupdates/aytumusicquiz/version.json';
        var this_version = localStorage.getItem('current_version') || 1;
        var now = (new Date()).getTime();

        $.getJSON(url, {ts: now, uptime: now - startup_time.getTime()}, function(data) {
            if(data.current_version > config.initial_version && data.current_version > this_version) {
                alert('Found new version! Updating, please wait...');
                loadScripts(data.location, data.current_version)
                .then(function() {
                    App.set('update_ready', true);
                });
            }
        });
    }


    var updateLoop = function() {
        setTimeout(function() {
            // Don't auto update stuff in browser
            //if (!window['EMULATED']) {
                checkUpdate();
            //}

            updateLoop();
        }, 10000);
    };
    updateLoop();



    function loadScripts(url_prefix, new_version) {
        url_prefix = url_prefix || '';

        var urls = [];
        var current_version = new_version || localStorage.getItem('current_version') || 0;

        current_version = parseInt(current_version, 10);

        function addUrl(url, is_css) {
            urls.push({
                url: url_prefix + url,
                key: url,
                unique: url + '_' + current_version,

                // Don't execute the new updates yet as it will interfere with whats currently running
                execute: is_css ? false : (new_version ? false : true)
            });
        }

        addUrl('index.html', true);
		addUrl('styles.css', true);

        addUrl('js/helpers.js');
        addUrl('js/components/loader.js');
        addUrl('js/components/modelmessage.js');
        addUrl('js/components/quiz.js');
        addUrl('js/components/quizgame.js');
        addUrl('js/components/quizcategories.js');
        addUrl('js/components/quizcategorytile.js');
        addUrl('js/components/quizanswer.js');
        addUrl('js/components/quizquestion.js');
        addUrl('js/components/quizlist.js');
        addUrl('js/components/questiontimer.js');
        addUrl('js/components/bonusround.js');
        addUrl('js/components/flptoolbar.js');
        addUrl('js/components/home.js');
        addUrl('js/components/community.js');
        addUrl('js/components/legal.js');
        addUrl('js/components/prizes.js');
        addUrl('js/components/scoreboard.js');
        addUrl('js/components/getflps.js');
        addUrl('js/components/toolbar.js');
        addUrl('js/components/toolbarapp.js');
        addUrl('js/components/user.js');
        addUrl('js/components/fullscreens/round3.js');
        addUrl('js/app.js');
		addUrl('js/config.js');

        if (new_version) {
            addUrl('js/launcher.js');
        }

        if (new_version)
            console.log('Updating scripts...');

        return basket.require.apply(basket, urls).then(function() {
            localStorage.setItem('current_version', current_version);
            if (new_version) {
                console.log('Updated scripts');
				alert('Update complete. Please close and re-launch the app now.');
            }
        });
    }
};